"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var Q = require('q');
var access_control_authorization_1 = require("./access-control.authorization");
var controller_utils_1 = require("../utils/controller.utils");
var entity_utils_1 = require("../utils/entity.utils");
var base_query_builder_1 = require("../utils/base.query-builder");
var AccessControlBaseController = /** @class */ (function () {
    function AccessControlBaseController(model, grants) {
        this.model = model;
        var _resource = model.collection.collectionName;
        grants.forEach(function (grant) { grant.resource = grant.resource || _resource; });
        this.ac = new access_control_authorization_1.CrudAccessControl(_resource, grants);
        logger_1.default.debug("inited access control for " + _resource);
    }
    AccessControlBaseController.prototype.create = function (req, res) {
        var self = this;
        self.ac.checkCreate(req)
            .then(function (check) {
            return self.entityFromBody(req)
                .then(check.setBeforeUpdate)
                .then(entity_utils_1.createEntity(self.model))
                .then(check.filterAfterQuery);
        })
            .then(controller_utils_1.respondWithResult(res))
            .catch(controller_utils_1.handleError(res));
    };
    AccessControlBaseController.prototype.find = function (req, res) {
        var self = this;
        self.ac.checkRead(req)
            .then(function (check) {
            return base_query_builder_1.createQueryExecutor(self.model, check.applyQueryRestriction)
                .then(base_query_builder_1.execFindAndCound(req.query, res))
                .then(check.filterAfterQuery);
        })
            .then(controller_utils_1.respondWithResult(res))
            .catch(controller_utils_1.handleError(res));
    };
    AccessControlBaseController.prototype.findById = function (req, res) {
        var self = this;
        self.ac.checkRead(req)
            .then(function (check) {
            return base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id, check.applyQueryRestriction)
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(check.filterAfterQuery);
        })
            .then(controller_utils_1.respondWithResult(res))
            .catch(controller_utils_1.handleError(res));
    };
    AccessControlBaseController.prototype.update = function (req, res) {
        var self = this;
        self.ac.checkUpdate(req)
            .then(function (check) {
            return base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id, check.applyQueryRestriction)
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(entity_utils_1.applyUpdate(req.body))
                .then(check.filterAfterQuery);
        })
            .then(controller_utils_1.respondWithResult(res))
            .catch(controller_utils_1.handleError(res));
    };
    AccessControlBaseController.prototype.patch = function (req, res) {
        var self = this;
        self.ac.checkUpdate(req)
            .then(function (check) {
            return base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id, check.applyQueryRestriction)
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(entity_utils_1.applyPatch(req.body))
                .then(check.filterAfterQuery);
        })
            .then(controller_utils_1.respondWithResult(res))
            .catch(controller_utils_1.handleError(res));
    };
    AccessControlBaseController.prototype.remove = function (req, res, model) {
        //contruct query
        var self = this;
        self.ac.checkUpdate(req)
            .then(function (check) {
            return base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id, check.applyQueryRestriction)
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(entity_utils_1.removeEntity())
                .then(controller_utils_1.successMessageResult());
        })
            .then(controller_utils_1.respondWithResult(res))
            .catch(controller_utils_1.handleError(res));
    };
    AccessControlBaseController.prototype.entityFromBody = function (req) {
        var entity = req.body;
        return Q.when(entity);
    };
    return AccessControlBaseController;
}());
exports.AccessControlBaseController = AccessControlBaseController;
