'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var accesscontrol_1 = require("accesscontrol");
var createError = require('http-errors');
var Q = require('q');
var CrudAccessControl = /** @class */ (function () {
    function CrudAccessControl(resource, grants) {
        this.resource = resource;
        this.grants = grants;
        this.ac = new accesscontrol_1.AccessControl(grants);
    }
    CrudAccessControl.prototype.checkCreate = function (req) {
        var self = this;
        return this.baseCheck(req, 'create', function (check, access) {
            return check;
        }, function (check, access) {
            logger_1.default.debug(req);
            check.setBeforeUpdate = self.setOwner(req);
            return check;
        });
    };
    CrudAccessControl.prototype.checkRead = function (req) {
        var self = this;
        return this.baseCheck(req, 'read', function (check, access) {
            return check;
        }, function (check, access) {
            check.applyQueryRestriction = self.restrictByUserOrOrganizationOwner(req);
            return check;
        });
    };
    CrudAccessControl.prototype.checkUpdate = function (req) {
        var self = this;
        return this.baseCheck(req, 'update', function (check, access) {
            return check;
        }, function (check, access) {
            check.applyQueryRestriction = self.restrictByUserOrOrganizationOwner(req);
            return check;
        });
    };
    CrudAccessControl.prototype.checkDelete = function (req) {
        return this.baseCheck(req, 'delete', function (check, access) {
            return check;
        }, function (check, access) {
            check.applyQueryRestriction = this.restrictByUserOrOrganizationOwner(req);
            return check;
        });
    };
    CrudAccessControl.prototype.baseCheck = function (req, op, accessAnyCallback, accessOwnCallback) {
        logger_1.default.debug(op + ":" + this.resource);
        logger_1.default.debug(req);
        logger_1.default.debug(req.user);
        logger_1.default.debug(req.user.roles);
        var defer = Q.defer();
        var query = this.ac.can(req.user.roles);
        var accessAny = query[op + 'Any'](this.resource);
        var check = new Check();
        if (accessAny.granted) {
            //any permission
            check.isGranted = true;
            check.filterAfterQuery = this.filterAfterQuery(accessAny);
            logger_1.default.trace('granted access any');
            defer.resolve(accessAnyCallback(check, accessAny));
        }
        else {
            //own permission
            var accessOwn = query[op + 'Own'](this.resource);
            if (accessOwn.granted) {
                check.isGranted = true;
                check.filterAfterQuery = this.filterAfterQuery(accessOwn);
                logger_1.default.trace('granted access own');
                defer.resolve(accessOwnCallback(check, accessOwn));
            }
        }
        if (!check.isGranted) {
            logger_1.default.warn('access denied.');
            //no permission at all
            defer.reject(createError(403, "user has no permission to " + op + " " + this.resource));
        }
        return defer.promise;
    };
    CrudAccessControl.prototype.setOwner = function (req) {
        return function (entity) {
            entity.owner = { userId: req.user._id };
            return entity;
        };
    };
    CrudAccessControl.prototype.filterAfterQuery = function (permission) {
        //return (toFilter) => permission.filter(toFilter);
        return function (toFilter) { return toFilter; };
    };
    CrudAccessControl.prototype.restrictByUserOrOrganizationOwner = function (req) {
        return function (query) {
            return query.where("owner.userId").eq(req.user._id);
            // return query.or([
            //   { "owner.userId" : req.user._id},
            //   { "owner.organizationId": req.user.organization_id },
            // ]);
        };
    };
    return CrudAccessControl;
}());
exports.CrudAccessControl = CrudAccessControl;
var Check = /** @class */ (function () {
    function Check(isGranted, applyQueryRestriction, setBeforeUpdate, filterAfterQuery) {
        this.isGranted = isGranted;
        this.applyQueryRestriction = applyQueryRestriction;
        this.setBeforeUpdate = setBeforeUpdate;
        this.filterAfterQuery = filterAfterQuery;
        function nop(param) {
            //l.debug(`nop: ${JSON.stringify(param)}`);
            return param;
        }
        this.isGranted = isGranted || false;
        this.applyQueryRestriction = nop;
        this.setBeforeUpdate = nop;
        this.filterAfterQuery = nop;
    }
    return Check;
}());
exports.Check = Check;
